function varargout = invv(varargin)
% INVV(A, sz)
% computes inverse of sz(k)-by-sz(k) systems using lapack.
% 
% The output is equivalent to 
%
%  p = cumsum([1, sz.2]);
%  for i=1:numel(sz),
%    a = reshape(A(p(i):p(i+1)-1), sz(i),sz(i));
%    B(p(i):p(i+1)-1) = reshape(inv(a), [], 1);
%  end

%{
Copyright 2009, 2010, 2011, 2012 SINTEF ICT, Applied Mathematics.

This file is part of The MATLAB Reservoir Simulation Toolbox (MRST).

MRST is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MRST is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MRST.  If not, see <http://www.gnu.org/licenses/>.
%}

% $Date: 2012-09-11 15:32:18 +0200 (Tue, 11 Sep 2012) $
% $Revision: 9662 $

% If we end up here, invertSmallMatrices needs to be built.
% Next time, this will not happen.
buildmex invv.c -lmwlapack

% Now, run it.
[varargout{1:nargout}] = invv(varargin{:});
